class PackageVersion:

    def __init__(self, version, sha256, arch, repository):
        self.version = version
        self.sha256 = sha256
        self.architecture = arch
        self.repository = repository
