class Update:

    def __init__(self, name, candidate_version, baseversion_hash):
        self.name = name
        self.candidateVersion = candidate_version
        self.baseVersionHash = baseversion_hash
